@@ -11,18 +11,15 @@ from dataclasses import dataclass, field
 import numpy as np
 
 from motulator.helpers import abc2complex, Bunch
-from motulator.control.common import SpeedCtrl, PWM
+from motulator.control.common import Ctrl, SpeedCtrl, PWM
 
 
 # %%
 @dataclass
 class InductionMotorVectorCtrlPars:
-    """
-    Vector control parameters for an induction motor drive.
+    """Vector control parameters for induction motor drives."""
 
-    """
     # pylint: disable=too-many-instance-attributes
-
     # Speed reference (in electrical rad/s)
     w_m_ref: Callable[[float], float] = field(
         repr=False, default=lambda t: (t > .2)*(2*np.pi*50))
@@ -34,6 +31,8 @@ class InductionMotorVectorCtrlPars:
     alpha_c: float = 2*np.pi*200
     alpha_o: float = 2*np.pi*40  # Used only in the sensorless mode
     alpha_s: float = 2*np.pi*4
+    # Sensored observer
+    g = .2  # Used only in the sensored mode
     # Maximum values
     tau_M_max: float = 1.5*14.6
     i_s_max: float = 1.5*np.sqrt(2)*5
@@ -50,7 +49,7 @@ class InductionMotorVectorCtrlPars:
 
 
 # %%
-class InductionMotorVectorCtrl:
+class InductionMotorVectorCtrl(Ctrl):
     """
     Vector control for an induction motor drive.
 
@@ -65,9 +64,8 @@ class InductionMotorVectorCtrl:
     """
 
     # pylint: disable=too-many-instance-attributes
-    def __init__(self, pars=InductionMotorVectorCtrlPars()):
-
-        self.t = 0
+    def __init__(self, pars):
+        super().__init__()
         self.T_s = pars.T_s
         self.w_m_ref = pars.w_m_ref
         self.sensorless = pars.sensorless
@@ -80,11 +78,10 @@ class InductionMotorVectorCtrl:
         else:
             self.observer = Observer(pars)
         self.pwm = PWM(pars)
-        self.data = Bunch()
 
     def __call__(self, mdl):
         """
-        Main control loop.
+        Run the main control loop.
 
         Parameters
         ----------
@@ -127,11 +124,21 @@ class InductionMotorVectorCtrl:
         u_s_ref = self.current_ctrl.output(i_s_ref, i_s)
         d_abc_ref, u_s_ref_lim = self.pwm.output(u_s_ref, u_dc, theta_s, w_s)
 
-        # Data logging
-        data = Bunch(i_s_ref=i_s_ref, i_s=i_s, u_s=u_s, w_m_ref=w_m_ref,
-                     w_m=w_m, w_s=w_s, psi_R=psi_R, theta_s=theta_s,
-                     u_dc=u_dc, tau_M_ref_lim=tau_M_ref_lim, t=self.t)
-        self._save(data)
+        # Save data
+        data = Bunch(
+            i_s=i_s,
+            i_s_ref=i_s_ref,
+            psi_R=psi_R,
+            t=self.t,
+            tau_M_ref_lim=tau_M_ref_lim,
+            theta_s=theta_s,
+            u_dc=u_dc,
+            u_s=u_s,
+            w_m=w_m,
+            w_m_ref=w_m_ref,
+            w_s=w_s,
+        )
+        self.save(data)
 
         # Update the states
         self.pwm.update(u_s_ref_lim)
@@ -139,22 +146,10 @@ class InductionMotorVectorCtrl:
         self.current_ref.update(u_s_ref, u_dc)
         self.current_ctrl.update(u_s_ref_lim, w_s)
         self.observer.update(i_s, w_s)
-        self.t += self.T_s
+        self.update_clock(self.T_s)
 
         return self.T_s, d_abc_ref
 
-    def _save(self, data):
-        for key, value in data.items():
-            self.data.setdefault(key, []).extend([value])
-
-    def post_process(self):
-        """
-        Transform the lists to the ndarray format.
-
-        """
-        for key in self.data:
-            self.data[key] = np.asarray(self.data[key])
-
 
 # %%
 class CurrentRef:
@@ -167,7 +162,7 @@ class CurrentRef:
 
     Parameters
     ----------
-    pars : InductionMotorVectorCtrlPars (or its subset)
+    pars : InductionMotorVectorCtrlPars
         Control parameters.
 
     Notes
@@ -183,7 +178,6 @@ class CurrentRef:
     """
 
     def __init__(self, pars):
-
         self.T_s = pars.T_s
         self.i_s_max = pars.i_s_max
         self.L_sgm = pars.L_sgm
@@ -219,6 +213,7 @@ class CurrentRef:
             Limited torque reference.
 
         """
+
         def q_axis_current_limit(i_sd_ref, psi_R):
             # Priority given to the d component
             i_sq_max1 = np.sqrt(self.i_s_max**2 - i_sd_ref**2)
@@ -228,17 +223,17 @@ class CurrentRef:
             i_sq_max = np.min([i_sq_max1, i_sq_max2])
             return i_sq_max
 
-        # Current reference
-        if psi_R > 0:
-            i_sq_ref = tau_M_ref/(1.5*self.p*psi_R)
-        else:
-            i_sq_ref = 0
+        # q-axis current reference
+        i_sq_ref = tau_M_ref/(1.5*self.p*psi_R) if psi_R > 0 else 0
+
         # Limit the current
         i_sq_max = q_axis_current_limit(self.i_sd_ref, psi_R)
         if np.abs(i_sq_ref) > i_sq_max:
             i_sq_ref = np.sign(i_sq_ref)*i_sq_max
+
         # Current reference
         i_s_ref = self.i_sd_ref + 1j*i_sq_ref
+
         # Limited torque (for the speed controller)
         tau_M_ref_lim = 1.5*self.p*psi_R*i_sq_ref
 
@@ -257,8 +252,8 @@ class CurrentRef:
 
         """
         u_s_max = u_dc/np.sqrt(3)
-        self.i_sd_ref += self.T_s*self.gain_fw*(u_s_max**2
-                                                - np.abs(u_s_ref)**2)
+        self.i_sd_ref += self.T_s*self.gain_fw*(
+            u_s_max**2 - np.abs(u_s_ref)**2)
         if self.i_sd_ref > self.i_sd_nom:
             self.i_sd_ref = self.i_sd_nom
         elif self.i_sd_ref < -self.i_s_max:
@@ -277,7 +272,7 @@ class CurrentCtrl:
 
     Parameters
     ----------
-    pars : InductionMotorVectorCtrlPars (or its subset)
+    pars : InductionMotorVectorCtrlPars
         Control parameters.
 
     Notes
@@ -293,7 +288,6 @@ class CurrentCtrl:
     """
 
     def __init__(self, pars):
-
         self.T_s = pars.T_s
         self.L_sgm = pars.L_sgm
         self.alpha_c = pars.alpha_c
@@ -359,7 +353,7 @@ class SensorlessObserver:
 
     Parameters
     ----------
-    pars : InductionMotorVectorCtrlPars (or its subset)
+    pars : InductionMotorVectorCtrlPars
         Control parameters.
 
     Notes
@@ -379,7 +373,6 @@ class SensorlessObserver:
 
     # pylint: disable=too-many-instance-attributes
     def __init__(self, pars):
-
         self.T_s = pars.T_s
         self.R_s = pars.R_s
         self.R_R = pars.R_R
@@ -402,8 +395,6 @@ class SensorlessObserver:
             Stator voltage in estimated rotor flux coordinates.
         i_s : complex
             Stator current in estimated rotor flux coordinates.
-        *_ : not used
-            Provides a compatible interface with the sensored observer.
 
         Returns
         -------
@@ -431,26 +422,20 @@ class SensorlessObserver:
             w_s = self.w_m
 
         # Slip angular frequency (stored for the update method)
-        if self.psi_R > 0:
-            self.w_r = e_r.imag/self.psi_R
-        else:
-            self.w_r = 0
+        self.w_r = e_r.imag/self.psi_R if self.psi_R > 0 else 0
 
         # Increment of the flux magnitude (stored for the update method)
-        self.dpsi_R = ((1 - g.real)*(e_s.real + w_s*self.L_sgm*i_s.imag)
-                       + g.real*e_r.real)
+        self.dpsi_R = ((1 - g.real)*(e_s.real + w_s*self.L_sgm*i_s.imag) +
+                       g.real*e_r.real)
 
         return w_s
 
     def update(self, i_s, w_s):
-        """
-        Update the states for the next sampling period.
-
-        """
+        """Update the states for the next sampling period."""
         self.w_m += self.T_s*self.alpha_o*(w_s - self.w_r)
         self.psi_R += self.T_s*self.dpsi_R
-        self.theta_s += self.T_s*w_s
-        self.theta_s = np.mod(self.theta_s, 2*np.pi)    # Limit to [0, 2*pi]
+        self.theta_s += self.T_s*w_s  # Next line: limit into [-pi, pi)
+        self.theta_s = np.mod(self.theta_s + np.pi, 2*np.pi) - np.pi
         self.i_s_old = i_s
 
 
@@ -465,18 +450,14 @@ class Observer:
 
     Parameters
     ----------
-    pars : InductionMotorVectorCtrlPars (or its subset)
+    pars : InductionMotorVectorCtrlPars
         Control parameters.
 
     Notes
     -----
     This implementation places the pole in synchronous coordinates at::
 
-        s = -R_R/L_M - g*abs(w_m) - 1j*w_s
-
-    The current model would be obtained using k = 1, resulting in the pole at
-    s = -R_R/L_M - 1j*(w_s - w_m). The pure voltage model corresponds to k = 0,
-    resulting in the marginally stable pole at s = -1j*w_s.
+        s = -R_R/L_M - g*abs(w_m) - 1j*(w_s - w_m)
 
     References
     ----------
@@ -488,14 +469,13 @@ class Observer:
 
     # pylint: disable=too-many-instance-attributes
     def __init__(self, pars):
-
         self.T_s = pars.T_s
         self.R_s = pars.R_s
         self.R_R = pars.R_R
         self.L_sgm = pars.L_sgm
-        self.L_M = pars.L_M
+        self.alpha = pars.R_R/pars.L_M
         # Nonnegative gain for damping
-        self.g = .5
+        self.g = pars.g
         # Initial states
         self.theta_s, self.psi_R, self.i_s_old = 0, 0, 0
         # Store for the update method to avoid recalculation
@@ -520,27 +500,24 @@ class Observer:
             Angular frequency of the rotor flux.
 
         """
-        alpha = self.R_R/self.L_M
-
         # The observer pole could be placed arbitrarily by means of the
-        # observer gain k. The following choice places the pole at
-        # s = -sigma - 1j*w_s, where sigma = alpha + g*abs(w_m). This allows
-        # smooth transition from the current model (at zero speed to the
-        # (damped) voltage model (at higher speeds).
-        k = (alpha + self.g*np.abs(w_m))/(alpha - 1j*w_m)
+        # observer gain k. The current model would be obtained using k = 1,
+        # resulting in the pole at s = -R_R/L_M - 1j*(w_s - w_m). The pure
+        # voltage model corresponds to k = 0, resulting in the marginally
+        # stable pole at s = -1j*w_s.
+
+        # This gain leads to s = -alpha - g*abs(w_m) - 1j*(w_s - w_m)
+        k = 1 + self.g*np.abs(w_m)/(self.alpha - 1j*w_m)
 
         # Induced voltage from the stator quantities
         e_s = u_s - self.R_s*i_s - self.L_sgm*(i_s - self.i_s_old)/self.T_s
         # Induced voltage from the rotor quantities
-        e_r = self.R_R*i_s - (alpha - 1j*w_m)*self.psi_R
+        e_r = self.R_R*i_s - (self.alpha - 1j*w_m)*self.psi_R
 
         # Angular frequency of the rotor flux vector
         den = self.psi_R + self.L_sgm*((1 - k)*i_s).real
         v = (1 - k)*e_s + k*e_r
-        if den > 0:
-            w_s = v.imag/den
-        else:
-            w_s = w_m
+        w_s = v.imag/den if den > 0 else w_m
 
         # Increment of the flux magnitude (stored for the update method)
         self.dpsi_R = v.real + w_s*self.L_sgm*((1 - k)*i_s).imag
@@ -548,11 +525,8 @@ class Observer:
         return w_s
 
     def update(self, i_s, w_s):
-        """
-        Update the states for the next sampling period.
-
-        """
+        """Update the states for the next sampling period."""
         self.psi_R += self.T_s*self.dpsi_R
-        self.theta_s += self.T_s*w_s
-        self.theta_s = np.mod(self.theta_s, 2*np.pi)    # Limit to [0, 2*pi]
+        self.theta_s += self.T_s*w_s  # Next line: limit into [-pi, pi)
+        self.theta_s = np.mod(self.theta_s + np.pi, 2*np.pi) - np.pi
         self.i_s_old = i_s
