@@ -230,7 +230,17 @@ class Simulation:
         simplicity, only max_step is included as an option of this method.
 
         """
-        # Simulation loop
+        try:
+            self.simulation_loop(t_stop, max_step)
+        except FloatingPointError:
+            print('Invalid value encountered at %.2f seconds.' % self.mdl.t0)
+        # Call the post-processing functions
+        self.mdl.post_process()
+        self.ctrl.post_process()
+
+    @np.errstate(invalid='raise')
+    def simulation_loop(self, t_stop, max_step):
+        """Run the main simulation loop."""
         while self.mdl.t0 <= t_stop:
 
             # Run the digital controller
@@ -264,10 +274,6 @@ class Simulation:
                     sol.q = len(sol.t)*[self.mdl.conv.q]
                     self.mdl.save(sol)
 
-        # Call the post-processing functions
-        self.mdl.post_process()
-        self.ctrl.post_process()
-
     def save_mat(self, name='sim'):
         """
         Save the simulation data into MATLAB .mat files.
